<?
/**
 * CSS Compressor
 * r:8 - May 13, 2009
 * Corey Hart @ http://www.codenothing.com
 */ 

// Include miscellaneous variables
include("vars/collect.php");


Class CSSCompression
{
	var $selectors = array();
	var $details = array();
	var $options = array();
	var $stats = array();
	var $media = false;

	/**
	 * Load the options according to user specifications with
	 * the class constructor
	 */ 
	function __construct(){
		// Converts long color names to short hex names (aliceblue -> #f0f8ff)
		$this->options['color-long2hex'] = $_POST['color-long2hex'];

		// Converts rgb colors to hex (rgb(159,80,98) -> #9F5062)
		$this->options['color-rgb2hex'] = $_POST['color-rgb2hex'];

		// Converts long hex codes to short color names (#f5f5dc -> beige)
		$this->options['color-hex2shortcolor'] = $_POST['color-hex2shortcolor'];

		// Converts long hex codes to short hex codes (#44ff11 -> #4f1)
		$this->options['color-hex2shorthex'] = $_POST['color-hex2shorthex'];

		// Converts font-weight names to numbers (bold -> 700)
		$this->options['fontweight2num'] = $_POST['fontweight2num'];

		// Removes zero decimals and 0 units (15.0px -> 15px || 0px -> 0)
		$this->options['format-units'] = $_POST['format-units'];

		// Lowercases html tags from list (BODY -> body)
		$this->options['lowercase-selectors'] = $_POST['lowercase-selectors'];

		// Compresses single defined multi-directional properties (margin: 15px 25px 15px 25px -> margin:15px 25px)
		$this->options['directional-compress'] = $_POST['directional-compress'];

		// Combines multiply defined selectors (p{color:blue;} p{font-size:12pt} -> p{color:blue;font-size:12pt;})
		$this->options['multiple-selectors'] = $_POST['multiple-selectors'];

		// Combines selectors with same details (p{color:blue;} a{color:blue;} -> p,a{color:blue;})
		$this->options['multiple-details'] = $_POST['multiple-details'];

		// Combines color/style/width properties (border-style:dashed;border-color:black;border-width:4px; -> border:4px dashed black)
		$this->options['csw-combine'] = $_POST['csw-combine'];

		// Combines cue/pause properties (cue-before: url(before.au); cue-after: url(after.au) -> cue:url(before.au) url(after.au))
		$this->options['auralcp-combine'] = $_POST['auralcp-combine'];

		// Combines margin/padding directionals (margin-top:10px;margin-right:5px;margin-bottom:4px;margin-left:1px; -> margin:10px 5px 4px 1px;)
		$this->options['mp-combine'] = $_POST['mp-combine'];

		// Combines border directionals (border-top|right|bottom|left:1px solid black -> border:1px solid black)
		$this->options['border-combine'] = $_POST['border-combine'];

		// Combines font properties (font-size:12pt; font-family: arial; -> font:12pt arial)
		$this->options['font-combine'] = $_POST['font-combine'];

		// Combines background properties (background-color: black; background-image: url(bgimg.jpeg); -> background:black url(bgimg.jpeg))
		$this->options['background-combine'] = $_POST['background-combine'];

		// Combines list-style properties (list-style-type: round; list-style-position: outside -> list-style:round outside)
		$this->options['list-combine'] = $_POST['list-combine'];

		// Removes multiply defined properties
		// STRONGLY SUGGESTED TO KEEP THIS TRUE
		$this->options['rm-multi-define'] = $_POST['rm-multi-define'];
	}

	function compress($css){
		// Start the timer
		$time = explode(' ', microtime());
		$this->stats['before']['time'] = $time[1] + $time[0];

		// Initial count for stats
		$this->stats['before']['size'] = strlen($css);

		// Send body through initial trimings
		$css = $this->initialTrim($css);

		// Seperate the element from the elements details
		$css = explode("\n", $css);
		foreach ($css as $details){
			// Determine whether your looking at the details or element
			if ($this->media && trim($details) == "}"){
				$MEDIA_STR .= "}\n";
				$this->media = false;
			}
			else if ($this->media){
				$MEDIA_STR .= trim($details);
			}
			else if (ereg("^{", $details)){
				unset($storage);
				$details = substr($details, 1, strlen($details)-2);
				$details = explode(";", $details);
				foreach($details as $line){
					if (eregi("^(url|@import)", $line)){
						$storage .= $line.";";
						continue;
					}
					list ($property, $value) = explode(":", $line);
					// Fail safe, remove unknown tag/elements
					if (!isset($property) || !isset($value)) continue;

					// Run the tag/element through each compression
					list ($property, $value) = $this->runSpecialCompressions($property, $value);

					// Add counter to before stats
					$this->stats['before']['props']++;

					// Store the compressed element
					$storage .= "$property:$value;";
				}
				// Store as the last known selector
				$this->details[$SEL_COUNTER] = $storage;
			}
			else if (eregi("^@import", trim($details))){
				// Seperate out each import string
				$arr = explode(";", $details);

				// Add to selector counter for details storage
				$SEL_COUNTER++;
				// Store the last entry as the selector
				$this->selectors[$SEL_COUNTER] = trim($arr[count($arr)-1]);

				// Clear out the last entry(the actual selector) and add to the import string
				unset($arr[count($arr)-1]);
				$IMPORT_STR .= trim(implode(';', $arr)).";";
			}
			else if (eregi("^@media", trim($details))){
				$this->media = true;
				$MEDIA_STR .= trim($details);
			}
			else if ($details){
				// Add to selector counter for details storage
				$SEL_COUNTER++;
				$this->selectors[$SEL_COUNTER] = trim($details);
			}

		}
		// Store the number of selectors before compression
		$this->stats['before']['selectors'] = count($this->selectors);

		// Compression Functions
		if ($this->options['lowercase-selectors']) 	$this->lowercaseSelectors();
		if ($this->options['multiple-selectors']) 	$this->combineMultiplyDefinedSelectors();
		if ($this->options['multiple-details']) 	$this->combineMuliplyDefinedDetails();
		foreach ($this->details as $key => $value){
			if ($this->options['csw-combine'])		$value = $this->combineCSWproperties($value);
			if ($this->options['auralcp-combine'])		$value = $this->combineAuralCuePause($value);
			if ($this->options['mp-combine']) 		$value = $this->combineMPproperties($value);
			if ($this->options['border-combine']) 		$value = $this->combineBorderDefinitions($value);
			if ($this->options['font-combine']) 		$value = $this->combineFontDefinitions($value);
			if ($this->options['background-combine']) 	$value = $this->combineBackgroundDefinitions($value);
			if ($this->options['list-combine']) 		$value = $this->combineListProperties($value);
			if ($this->options['rm-multi-define']) 		$value = $this->removeMultipleDefinitions($value);

			// Restore the value
			$this->details[$key] = $value;
		}

		// Run final stats
		$this->runFinalStatistics();

		// Format css to users preference
		$css =  $this->readability($IMPORT_STR, intval($_POST['readability']));

		// Add media string with comments to compress seperately
		if ($MEDIA_STR){
			$this->media = true;
			$css = "/** Media Types are not compressed with this script, cut out and compress each section seperately **/"
				."\n$MEDIA_STR\n\n/** The rest of your CSS File **/\n$css";
		}

		// Final count for stats
		$this->stats['after']['size'] = strlen($css);

		// Compression time
		$time = explode(' ', microtime());
		$this->stats['after']['time'] = $time[1] + $time[0];

		// Return compressed css
		return $css;
	}

	function initialTrim($css){
		// Regex
		$search = array(
			1 => "(\r|\n|\t)is", // Move extraneous spaces
			2 => "((\/\*|\<\!\-\-)(.*?)(\*\/|\-\-\>))is", // Remove all comments
			3 => "(\s{2,})is", // Remove multiple spaces
			4 => "(\s{0,}([,{};:>+])\s{0,})is", // Remove un-needed spaces around special characters
			5 => "(url\(['\"](.*?)['\"]\))is", // Remove quotes from urls
			6 => "(;{2,})is", // Remove unecessary semi-colons
			// Leave section open for additional entries

			// Break apart elements for setup of further compression
			20 => "(\{)is",
			21 => "(\})is",
		);

		// Replacements
		$replace = array(
			1 => " ",
			2 => " ",
			3 => " ",
			4 => "$1",
			5 => "url($1)",
			6 => ";",
			// Leave section open for additional entries

			// Add new line for setup of further compression
			20 => "\n{",
			21 => "}\n",
		);

		// Run replacements
		return preg_replace($search, $replace, $css);
	}

	function runSpecialCompressions($prop, $val){
		global $fontweight2num;
		// Properties should always be lowercase
		$prop = strtolower($prop);

		// Remove uneeded side definitions if possible
		if ($this->options['directional-compress'] && eregi("^(margin|padding|border|outline)", $prop)){
			$val = $this->sidesDirectional($val);
		}

		// Font-weight converter
		if ($this->options['fontweight2num'] && eregi("^font-weight$", $prop)){
			if (isset($fontweight2num[strtolower($val)]))
				$val = $fontweight2num[strtolower($val)];
		}

		// Remove uneeded decimals/units
		if ($this->options['format-units']){
			$val = $this->removeDecimal($val);
			$val = $this->removeUnits($val);
		}

		// Seperate out multi-definitions if possible
		$arr = explode(" ", $val);
		foreach ($arr as $k=>$v) $arr[$k] = $this->runColorChanges($v);
		$val = trim(implode(" ", $arr));

		// Return for list retrival
		return array($prop, $val);
	}

	function sidesDirectional($val){
		// Side definitions already reduced down to a single definition
		if (!ereg(" ", $val)){
			// Redundent, but just in case
			if ($this->options['format-units']){
				$val = $this->removeDecimal($val);
				$val = $this->removeUnits($val);
			}
			return $val;
		}

		// Split up each definiton
		$direction = explode(" ", $val);

		// Zero out and remove units if possible
		if ($this->options['format-units']){
			foreach ($direction as $k=>$v) $direction[$k] = $this->removeDecimal($v);
			foreach ($direction as $k=>$v) $direction[$k] = $this->removeUnits($v);
		}

		// 4 Direction reduction
		if (count($direction) == 4){
			if ($direction[0] == $direction[1] && $direction[2] == $direction[3] && $direction[0] == $direction[3]){
				// All 4 sides are the same, combine into 1 definition
				$val = $direction[0];
			}
			else if ($direction[0] == $direction[2] && $direction[1] == $direction[3]){
				// top-bottom/left-right are the same, reduce definition
				$val = $direction[0]." ".$direction[1];
			}
			else{
				// No reduction found, return in initial form
				$val = implode(" ", $direction);
			}
		}
		// 2 Direction reduction
		else if (count($direction) == 2){
			if ($direction[0] == $direction[1]){
				// Both directions are the same, combine into single definition
				$val = $direction[0];
			}else{
				// No reduction found, return in initial form
				$val = implode(" ", $direction);
			}
		}
		// No reduction found, return in initial form
		else{
			$val = implode(" ", $direction);
		}

		// Return the value of the property
		return $val;
	}

	function removeDecimal($str){
		// Find all instances of .0 and remove them
		$pattern = "/^(\d+\.0)(\%|\w{2})/i";
		preg_match_all($pattern, $str, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			$search = "(".$matches[0][$i].")is";
			$replace = intval($matches[1][$i]).$matches[2][$i];
			$str = preg_replace($search, $replace, $str);
		}
		return $str;
	}

	function removeUnits($str){
		// Find all instants of 0 size and remove suffix
		$pattern = "/^(\d)(\%|\w{2})/i";
		preg_match_all($pattern, $str, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			if (intval($matches[1][$i]) == 0){
				$search = "(".$matches[0][$i].")is";
				$replace = "0";
				$str = preg_replace($search, $replace, $str);
			}
		}
		return $str;
	}

	function runColorChanges($val){
		global $long2hex, $hex2short;
		// Transfer rgb colors to hex codes
		if ($this->options['color-rgb2hex']){
			$pattern = "/rgb\((\d{1,3}(,\s?\d{1,3},\s?\d{1,3})?)\)/i";
			preg_match_all($pattern, $val, $matches);
			for ($i=0; $i<count($matches[1]); $i++){
				unset($new, $str);
				$hex = "0123456789ABCDEF";
				$str = explode(",", $matches[1][$i]);
				// Incase rgb was defined with single val
				if (!$str) $str = array($matches[1][$i]);
				foreach($str as $x){
					$x = intval($x);
					if ($x > 255) $x = 255;
					if ($x < 0) $x = 0;
					$new .= $hex[($x-$x%16)/16];
					$new .= $hex[$x%16];
				}
				// Repeat hex code to complete 6 digit hex requirement for single definitions
				if (count($str) == 1) $new .= $new.$new;
				// Escape out brackets
				$matches[0][$i] = str_replace("(", "\(", $matches[0][$i]);
				$matches[0][$i] = str_replace(")", "\)", $matches[0][$i]);
				// Replace within string
				$val = preg_replace("(".$matches[0][$i].")is", "#$new", $val);
			}
		}

		// Convert long color names to hex codes
		if ($this->options['color-long2hex']){
			foreach ($long2hex as $x=>$y){
				if (strtolower($val) == $x) $val = $y;
			}
		}

		// Convert 6 digit hex codes to short color names
		if ($this->options['color-hex2shortcolor']){
			foreach($hex2short as $x=>$y){
				if (strtolower($val) == $x) $val = $y;
			}
		}

		// Convert large hex codes to small codes
		if ($this->options['color-hex2shorthex']){
			$pattern = "/#([0-9a-fA-F]{6})/i";
			preg_match_all($pattern, $val, $matches);
			for ($i=0; $i<count($matches[1]); $i++){
				$hex = str_split($matches[1][$i], 1);
				if ($hex[0] == $hex[1] && $hex[2] == $hex[3] && $hex[4] == $hex[5]){
					$search = "(".$matches[0][$i].")is";
					$replace = "#".$hex[0].$hex[2].$hex[4];
					$val = preg_replace($search, $replace, $val);
				}
			}
		}

		// Return transformed value
		return $val;
	}

	function lowercaseSelectors(){
		foreach ($this->selectors as $k=>$v){
			$a_arr = explode(' ', $v);
			foreach ($a_arr as $a_key=>$a_val){
				if (eregi(">", $a_val)){
					$b_arr = explode(">", $a_val);
					foreach ($b_arr as $b_key=>$b_val){
						if (eregi("\+", $b_val)){
							$c_arr = explode("+", $b_val);
							foreach ($c_arr as $c_key=>$c_val){
								$c_arr[$c_key] = $this->lowerSelectorExtras($c_val);
							}
							$b_arr[$b_key] = trim(implode("+", $c_arr));
						}else{
							$b_arr[$b_key] = $this->lowerSelectorExtras($b_val);
						}
					}
					$a_arr[$a_key] = trim(implode(">", $b_arr));
				}
				else if (eregi("\+", $a_val)){
					$b_arr = explode("+", $a_val);
					foreach ($b_arr as $b_key=>$b_val){
						$b_arr[$b_key] = $this->lowerSelectorExtras($b_val);
					}
					$a_arr[$a_key] = trim(implode("+", $b_arr));
				}
				else{
					$a_arr[$a_key] = $this->lowerSelectorExtras($a_val);
				}
			}
			$this->selectors[$k] = trim(implode(' ', $a_arr));
		}
	}

	function lowerSelectorExtras($val){
		if (eregi(":", $val)){
			list ($sel, $pseudo) = explode(":", $val);
			if (eregi("^[a-zA-Z]+$", $sel)) $sel = strtolower($sel);
			return "$sel:".strtolower($pseudo);
		}
		else if (eregi("^[a-zA-Z]+$", $val)){
			return strtolower($val);
		}
		else{
			return $val;
		}
	}

	function combineMultiplyDefinedSelectors(){
		$max = array_pop(array_keys($this->selectors))+1;
		for ($i=0; $i<$max; $i++){
			if (!$this->selectors[$i]) continue;
			for ($k=$i+1; $k<$max; $k++){
				if (!$this->selectors[$k]) continue;
				if ($this->selectors[$i] == $this->selectors[$k]){
					$this->details[$i] .= $this->details[$k];
					unset($this->selectors[$k], $this->details[$k]);
				}
			}
		}
	}

	function combineMuliplyDefinedDetails(){
		$max = array_pop(array_keys($this->selectors))+1;
		for ($i=0; $i<$max; $i++){
			if (!$this->selectors[$i]) continue;
			$arr = explode(";", $this->details[$i]);
			for ($k=$i+1; $k<$max; $k++){
				if (!$this->selectors[$k]) continue;
				$match = explode(";", $this->details[$k]);
				$x = array_diff($arr, $match);
				$y = array_diff($match, $arr);
				if (count($x) < 1 && count($y) < 1){
					$this->selectors[$i] .= ",".$this->selectors[$k];
					unset($this->details[$k], $this->selectors[$k]);
				}
			}
		}
	}

	function combineCSWproperties($val){
		$storage = array();
		$pattern = "/(border|outline)-(color|style|width):(.*?);/i";
		preg_match_all($pattern, $val, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			$storage[strtolower($matches[1][$i])][strtolower($matches[2][$i])] = $matches[3][$i];
		}

		// Go through each tag for possible combination
		foreach($storage as $tag => $arr){
			// Make sure all 3 are defined and they aren't directionals
			if (count($arr) == 3 && !$this->checkUncombinables($arr)){
				// String to replace each instance with
				$replace = "$tag:".$arr['width']." ".$arr['style']." ".$arr['color'];
				foreach ($arr as $x=>$y){
					// Replace every instance, as multiple declarations removal will correct it
					$search = "($tag-$x:$y)is";
					$val = preg_replace($search, $replace, $val);
				}
			}
		}

		// Return value
		return $val;
	}

	function combineAuralCuePause($val){
		$storage = array();
		$pattern = "/(cue|pause)-(before|after):(.*?);/i";
		preg_match_all($pattern, $val, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			$storage[strtolower($matches[1][$i])][strtolower($matches[2][$i])] = $matches[3][$i];
		}

		// Go through each tag for possible combination
		foreach($storage as $tag => $arr){
			if (count($arr) == 2 && !$this->checkUncombinables($arr)){
				// String to replace each instance with
				$replace = "$tag:".$arr['before']." ".$arr['after'];
				foreach ($arr as $x=>$y){
					// Escape out url parameters for searches
					$y = str_replace("(", "\(", $y);
					$y = str_replace(")", "\)", $y);
					// Replace every instance, as multiple declarations removal will correct it
					$search = "($tag-$x:$y)is";
					$val = preg_replace($search, $replace, $val);
				}
			}
		}

		// Return value
		return $val;
	}

	function combineMPproperties($val){
		$storage = array();
		$pattern = "/(margin|padding)-(top|right|bottom|left):(.*?);/i";
		preg_match_all($pattern, $val, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			if (!isset($storage[$matches[1][$i]])) $storage[$matches[1][$i]] = array($matches[2][$i] => $matches[3][$i]);
			// Override double written properties
			$storage[$matches[1][$i]][$matches[2][$i]] = $matches[3][$i];
		}

		// Go through each tag for possible combination
		foreach($storage as $tag => $arr){
			// Drop capitols
			$tag = strtolower($tag);
			// Only combine if all 4 definitions are found
			if (count($arr) == 4 && !$this->checkUncombinables($arr)){
				// If all definitions are the same, combine into single definition
				if ($arr['top'] == $arr['bottom'] && $arr['left'] == $arr['right'] && $arr['top'] == $arr['left']){
					// String to replace each instance with
					$replace = "$tag:".$arr['top'];
					foreach ($arr as $a=>$b){
						// Replace every instance, as multiple declarations removal will correct it
						$search = "($tag-$a:$b)is";
						$val = preg_replace($search, $replace, $val);
					}
				}
				// If opposites are the same, combine into single definition
				else if ($arr['top'] == $arr['bottom'] && $arr['left'] == $arr['right']){
					// String to replace each instance with
					$replace = "$tag:".$arr['top']." ".$arr['left'];
					foreach ($arr as $a=>$b){
						// Replace every instance, as multiple declarations removal will correct it
						$search = "($tag-$a:$b)is";
						$val = preg_replace($search, $replace, $val);
					}
				}
				else{
					// String to replace each instance with
					$replace = "$tag:".$arr['top']." ".$arr['right']." ".$arr['bottom']." ".$arr['left'];
					foreach ($arr as $a=>$b){
						// Replace every instance, as multiple declarations removal will correct it
						$search = "($tag-$a:$b)is";
						$val = preg_replace($search, $replace, $val);
					}
				}
			}
		}

		// Return Value
		return $val;
	}

	function combineBorderDefinitions($val){
		$storage = array();
		$pattern = "/(border)-(top|right|bottom|left):(.*?);/i";
		preg_match_all($pattern, $val, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			if (!isset($storage[$matches[1][$i]])) $storage[$matches[1][$i]] = array($matches[2][$i] => $matches[3][$i]);
			// Override double written properties
			$storage[$matches[1][$i]][$matches[2][$i]] = $matches[3][$i];
		}

		foreach ($storage as $tag => $arr){
			if (count($arr) == 4 && $arr['top'] == $arr['bottom'] && $arr['left'] == $arr['right'] && $arr['top'] == $arr['right']){
				// String to replace each instance with
				$replace = "$tag:".$arr['top'];
				foreach ($arr as $a=>$b){
					// Replace every instance, as multiple declarations removal will correct it
					$search = "($tag-$a:$b)is";
					$val = preg_replace($search, $replace, $val);
				}
			}
		}

		// Return Value
		return $val;
	}

	function combineFontDefinitions($val){
		$storage = array();
		$pattern = "/(font|line)-(style|variant|weight|size|height|family):(.*?);/i";
		preg_match_all($pattern, $val, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			// Store each property in it's full state
			$storage[$matches[1][$i]."-".$matches[2][$i]] = $matches[3][$i];
		}

		// Combine font-size & line-height if possible
		if (isset($storage['font-size']) && isset($storage['line-height'])){
			$storage['size/height'] = $storage['font-size']."/".$storage['line-height'];
			unset($storage['font-size'], $storage['line-height']);
		}

		// Run font checks and get replacement str
		$replace = $this->searchDefinitions("font", $storage, array("font-style", "font-variant", "font-weight", "size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-style", "font-variant", "font-weight", "font-size", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-style", "font-variant", "size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-style", "font-variant", "font-size", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-style", "font-weight", "size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-style", "font-weight", "font-size", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-variant", "font-weight", "size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-variant", "font-weight", "font-size", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-weight", "size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-weight", "font-size", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-variant", "size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-variant", "font-size", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-style", "size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-style", "font-size", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("size/height", "font-family"));
		if (!$replace) $replace = $this->searchDefinitions("font", $storage, array("font-size", "font-family"));

		// If replacement string found, run it on all options
		if ($replace){
			for ($i=0; $i<count($matches[1]); $i++){
				if (!isset($storage['line-height']) || (isset($storage['line-height']) && !eregi("^line-height", $matches[0][$i]))){
					$val = preg_replace("(".$matches[0][$i].")is", $replace, $val);
				}
			}
		}

		// Return Value
		return $val;
	}

	function combineBackgroundDefinitions($val){
		$storage = array();
		$pattern = "/background-(color|image|repeat|attachment|position):(.*?);/i";
		preg_match_all($pattern, $val, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			// Store each property in it's full state
			$storage[$matches[1][$i]] = $matches[2][$i];
		}

		// Run background checks and get replacement str
		// With color
		$replace = $this->searchDefinitions("background", $storage, array("color", "image", "repeat", "attachment", "position"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color", "image", "attachment", "position"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color", "image", "repeat", "position"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color", "image", "repeat", "attachment"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color", "image", "repeat"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color", "image", "attachment"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color", "image", "position"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color", "image"));
		// Without Color
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("image", "attachment", "position"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("image", "repeat", "position"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("image", "repeat", "attachment"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("image", "repeat"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("image", "attachment"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("image", "position"));
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("image"));
		// Just Color
		if (!$replace) $replace = $this->searchDefinitions("background", $storage, array("color"));

		// If replacement string found, run it on all options
		if ($replace){
			for ($i=0; $i<count($matches[1]); $i++){
				// Escape out url braces
				$matches[0][$i] = str_replace("(", "\(", $matches[0][$i]);
				$matches[0][$i] = str_replace(")", "\)", $matches[0][$i]);
				$val = preg_replace("(".$matches[0][$i].")is", $replace, $val);
			}
		}

		// Return Value
		return $val;
	}

	function combineListProperties($val){
		$storage = array();
		$pattern = "/list-style-(type|position|image):(.*?);/i";
		preg_match_all($pattern, $val, $matches);
		for ($i=0; $i<count($matches[1]); $i++){
			// Store secondhand prop
			$storage[$matches[1][$i]] = $matches[2][$i];
		}

		// Run search patterns for replacement string
		$replace = $this->searchDefinitions("list-style", $storage, array("type", "position", "image"));
		if (!$replace) $replace = $this->searchDefinitions("list-style", $storage, array("type", "position"));
		if (!$replace) $replace = $this->searchDefinitions("list-style", $storage, array("type", "image"));
		if (!$replace) $replace = $this->searchDefinitions("list-style", $storage, array("position", "image"));
		if (!$replace) $replace = $this->searchDefinitions("list-style", $storage, array("type"));
		if (!$replace) $replace = $this->searchDefinitions("list-style", $storage, array("position"));
		if (!$replace) $replace = $this->searchDefinitions("list-style", $storage, array("image"));

		// If replacement string found, run it on all options
		if ($replace){
			for ($i=0; $i<count($matches[1]); $i++){
				// Escape out url braces
				$matches[0][$i] = str_replace("(", "\(", $matches[0][$i]);
				$matches[0][$i] = str_replace(")", "\)", $matches[0][$i]);
				$val = preg_replace("(".$matches[0][$i].")is", $replace, $val);
			}
		}

		// Return Value
		return $val;
	}

	// Returns true if special definitions are found
	function checkUncombinables($obj){
		if (is_array($obj)){
			foreach ($obj as $item){
				if (eregi("inherit|\!important| ", $item)) return true;
			}
			return false;
		}else{
			return eregi("inherit|\!important| ", $obj);
		}
	}

	function searchDefinitions($prop, $storage, $search){
		$str = "$prop:";
		foreach ($search as $value){
			if (!isset($storage[$value]) || $this->checkUncombinables($storage[$value])) return false;
			$str .= $storage[$value]." ";
		}
		return trim($str).";";
	}

	function removeMultipleDefinitions($val){
		$storage = array();
		$arr = explode(";", $val);
		foreach($arr as $x){
			if ($x){
				list($a, $b) = explode(":", $x);
				$storage[$a] = $b;
			}
		}
		if ($storage){
			unset($val);
			foreach($storage as $x=>$y){
				$val .= "$x:$y;";
			}
		}

		// Return Value
		return $val;
	}

	function runFinalStatistics(){
		// Selectors and props
		$this->stats['after']['selectors'] = count($this->selectors);
		foreach ($this->details as $item){
			$props = explode(";", $item);
			// Make sure count is true
			foreach ($props as $k=>$v){
				if (!isset($v) || $v == "") unset($props[$k]);
			}
			$this->stats['after']['props'] += count($props);
		}
	}

	function readability($import, $read){
		if ($read == 3){
			$css = str_replace(";", ";\n", $import);
			if ($import) $css .= "\n";
			foreach ($this->selectors as $k=>$v){
				$v = str_replace(">", " > ", $v);
				$v = str_replace("+", " + ", $v);
				$v = str_replace(",", ", ", $v);
				$css .= "$v {\n";
				$arr = explode(";", $this->details[$k]);
				foreach ($arr as $item){
					if (!$item) continue;
					list ($prop, $val) = explode(":", $item);
					$css .= "\t$prop: $val;\n";
				}
				$css .= "}\n\n";
			}
		}
		else if ($read == 2){
			$css = str_replace(";", ";\n", $import);
			foreach ($this->selectors as $k=>$v){
				$css .= "$v {\n\t".$this->details[$k]."\n}\n";
			}
		}
		else if ($read == 1){
			$css = str_replace(";", ";\n", $import);
			foreach ($this->selectors as $k=>$v){
				$css .= "$v{".$this->details[$k]."}\n";
			}
		}
		else{
			$css = $import;
			foreach ($this->selectors as $k=>$v){
				$css .= trim("$v{".$this->details[$k]."}");
			}
		}

		// Return formatted script
		return trim($css);
	}

	function displayStats(){
		// Set before/after arrays
		$before = $this->stats['before'];
		$after = $this->stats['after'];

		// Calc final size
		$size = $before['size']-$after['size'];

		// Display the table
		echo "<table cellspacing='1' cellpadding='2' style='width:400px;margin-bottom:20px;'>
			<tr bgcolor='#d1d1d1' align='center'>
				<th bgcolor='white' style='color:#8B0000;'>Results &raquo;</th>
				<th width='100'>Before</th>
				<th width='100'>After</th>
				<th width='100'>Compresssion</th>
			</tr>
			<tr bgcolor='#f1f1f1' align='center'>
				<th bgcolor='#d1d1d1'>Time</th>
				<td>-</td>
				<td>-</td>
				<td>".round($after['time']-$before['time'],2)." seconds</td>
			</tr>
			<tr bgcolor='#f1f1f1' align='center'>
				<th bgcolor='#d1d1d1'>Selectors</th>
				<td>".$before['selectors']."</td>
				<td>".$after['selectors']."</td>
				<td>".($before['selectors']-$after['selectors'])."</td>
			</tr>
			<tr bgcolor='#f1f1f1' align='center'>
				<th bgcolor='#d1d1d1'>Properties</th>
				<td>".$before['props']."</td>
				<td>".$after['props']."</td>
				<td>".($before['props']-$after['props'])."</td>
			</tr>
			<tr bgcolor='#f1f1f1' align='center'>
				<th bgcolor='#d1d1d1'>Size</th>
				<td>".$this->displaySizes($before['size'])."</td>
				<td>".$this->displaySizes($after['size'])."</td>
				<td>".$this->displaySizes($before['size']-$after['size'])."</td>
			</tr>
			</table>";
	}

	function displaySizes($size){
		$ext = array('B', 'K', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');
		for($c=0; $size>1024; $c++) $size /= 1024;
		return round($size,2).$ext[$c];
	}
};

/* Create the object */
$CSSC = new CSSCompression;
?>
